@@ -10,14 +10,10 @@
 /* .Call calls */
 extern SEXP R_pnormMRS(SEXP);
 extern SEXP R_inner(SEXP, SEXP);
-extern SEXP R_ltmatrices_solve(SEXP, SEXP, SEXP, SEXP, SEXP);
-extern SEXP R_ltmatrices_tcrossprod(SEXP, SEXP, SEXP, SEXP, SEXP);
 
 static const R_CallMethodDef CallEntries[] = {
     {"R_pnormMRS", (DL_FUNC) &R_pnormMRS, 1},
     {"R_inner", (DL_FUNC) &R_inner, 2},
-    {"R_ltmatrices_solve", (DL_FUNC) &R_ltmatrices_solve, 5},
-    {"R_ltmatrices_tcrossprod", (DL_FUNC) &R_ltmatrices_tcrossprod, 5},
     {NULL, NULL, 0}
 };
 
